#ifndef _PATH_WIDGET_H_
#define _PATH_WIDGET_H_
#include <vector>

#include <boost/shared_ptr.hpp>

#include <QDir>
#include <QScrollArea>
#include <QSize>
#include <QWidget>

#include "alg/sequence.hpp"
#include "alg/glsequence.hpp"
#include "qui/seqbrowser/ScrollableSequenceBrowser.hpp"
#include "qui/seqbrowser/SequenceBrowserSidebar.hpp"

//! the master widget that makes a useful Sequence Browser
class SequenceBrowserWidget : public QScrollArea 
{
  Q_OBJECT 

public:
  SequenceBrowserWidget(boost::shared_ptr<QDir>, QWidget *parent=0);

  //! return the popup menu for the glcanvas (reference stored internally)
  QMenu *getPopupMenu();
  //! return our fasta copy action (reference stored internally)
  QAction *getCopySelectedSequenceAsFastaAction();
  //! return our string copy action (reference stored internally)
  QAction *getCopySelectedSequenceAsStringAction();
  //! return sequence properties edit action
  QAction *getEditSequencePropertiesAction();

  QSize sizeHint() const;
    
  //! copy selected track regions into a list of SequenceLocations
  void copySelectedTracksAsSeqLocation(std::list<SequenceLocation>& );

  //! return current zoom size
  double zoom();

  //! reset all of our stored data back to the empty state
  void clear();

  /* Attach sequences to our browser, its best to attach them all
   * at once, as otherwise the Sidebar has some rendering trouble
   * as it draw and redraws the SequenceDescriptions in the various
   * positions.
   */
  void setSequences(
      const std::vector< boost::shared_ptr<Sequence> >& sequences, 
      boost::shared_ptr<AnnotationColors> cm);
  void setSequences(std::vector<boost::shared_ptr<GlSequence> >& sequences);
  const std::vector<boost::shared_ptr<GlSequence> >& sequences() const;

  //! reset just the links we're displaying
  void clear_links();
  //! set per species links
  void link(const std::vector<int> &path, const std::vector<bool>& isRC, int length);
  // return set of pathids defined by order of link calls
  const std::set<int> selectedPaths() const;
  void centerOnPath(const std::vector<int>& paths);
 
public slots:
  //! copy selected sequence
  void copySelectedSequenceAsFasta();

  //! set the zoom level of our browser
  void setZoom(double);
  //! zoom to fit the whole scene on the screen
  double zoomOut();
  //! zoom to sequence
  double zoomToSequence();

  void updatePosition();
  //! ask the user where to save an image of the current browser view
  void promptSaveBrowserPixmap();

  void update();
  
signals:
  //! emit when someone copies basepairs
  void basepairsCopied(size_t);
  
private:
  boost::shared_ptr<QDir> default_dir;
  ScrollableSequenceBrowser *scrollable_browser;
  SequenceBrowserSidebar *left_sidebar;
  SequenceBrowserSidebar *right_sidebar;
  QHBoxLayout *layout;
  QWidget *layout_container;

  //! sequences created by a setSequences(vector<Sequences>) call
  /*! I need to save them so i can free them to avoid a memory leak 
   */
  std::vector<boost::shared_ptr<GlSequence> > converted_sequences;
};
#endif
